# 🚀 项目名称 xinkeaboard-admin

> 基于 [UmiJS](https://umijs.org/) 和 Ant Design 的中后台管理系统，支持动态主题切换、高性能构建与模块化开发。

---

## 📦 技术栈

- **框架**：UmiJS
- **UI 组件**：Ant Design
- **语言**：TypeScript
- **样式**：Less
- **包管理器**：pnpm 10.12.1+
- **运行环境**：Node.js 18.20.4+

---

## ⚙️ 环境准备

建议使用 [nvm](https://github.com/nvm-sh/nvm) 管理 Node.js 版本：

```bash
nvm install 18.20.4
nvm use 18.20.4
```

安装 pnpm：

```bash
npm install -g pnpm@10.12.1
```

---

## 🚀 快速开始


### 1. 安装依赖

```bash
pnpm install
```

### 3. 启动开发环境

```bash
pnpm start
```

访问：http://localhost:8107

---

## 🛠️ 构建与预览

### 构建生产版本

```bash
pnpm build
```

---

## 📁 项目结构

```
├── public/               # 公共资源
├── src/                  # 源码目录
│   ├── assets/           # 图片、图标等资源
│   ├── components/       # 通用组件
│   ├── layouts/          # 页面布局
│   ├── models/           # 状态模型（可选）
│   ├── pages/            # 页面目录
│   ├── services/         # 接口请求
│   ├── styles/           # 全局样式
│   ├── app.tsx           # 应用入口
│   └── global.less       # 全局 less
├── .umirc.ts             # Umi 配置文件
├── package.json
├── tsconfig.json
└── README.md
```

---

## 🧠 常见问题

### 1. 启动或构建内存不足报错？

可以提升 Node 内存限制：

```bash
export NODE_OPTIONS="--max_old_space_size=6666"
```

或者在 `package.json` 中配置：

```json
"scripts": {
      "start": "cross-env APP_TYPE=site NODE_OPTIONS=--max_old_space_size=6666 umi dev --port=8107",
}
```

### 2. 让 Node 使用 OpenSSL 旧版兼容模式，避免新版本加密错误


```bash
export NODE_OPTIONS="--openssl-legacy-provider"
```

或者在 `package.json` 中配置：

```json
"scripts": {
      "start": "cross-env APP_TYPE=site NODE_OPTIONS=--openssl-legacy-provider umi dev --port=8107",
}
```
