package com.slodon.b2b2c.core.util;

import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @description: 导出通用工具类
 * @author: wuxy
 * @create: 2019-07-08 10:22
 **/
public class ExcelUtil {

    public static byte[] export(String sheetTitle, String[] title, List<?> list) {
        //创建excel表
        HSSFWorkbook wb = new HSSFWorkbook();
        HSSFSheet sheet = wb.createSheet(sheetTitle);
        //设置默认行宽
        sheet.setDefaultColumnWidth(20);

        //表头样式（加粗，水平居中，垂直居中）
        HSSFCellStyle cellStyle = wb.createCellStyle();
        //水平居中
        cellStyle.setAlignment(HSSFCellStyle.ALIGN_CENTER);
        //垂直居中
        cellStyle.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        //设置边框样式
        //下边框
        cellStyle.setBorderBottom(HSSFCellStyle.BORDER_THIN);
        //左边框
        cellStyle.setBorderLeft(HSSFCellStyle.BORDER_THIN);
        //上边框
        cellStyle.setBorderTop(HSSFCellStyle.BORDER_THIN);
        //右边框
        cellStyle.setBorderRight(HSSFCellStyle.BORDER_THIN);

        HSSFFont fontStyle = wb.createFont();
        fontStyle.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);

        cellStyle.setFont(fontStyle);

        //标题样式（加粗，垂直居中）
        HSSFCellStyle cellStyle2 = wb.createCellStyle();
        //垂直居中
        cellStyle2.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        cellStyle2.setFont(fontStyle);

        //设置边框样式
        //下边框
        cellStyle2.setBorderBottom(HSSFCellStyle.BORDER_THIN);
        //左边框
        cellStyle2.setBorderLeft(HSSFCellStyle.BORDER_THIN);
        //上边框
        cellStyle2.setBorderTop(HSSFCellStyle.BORDER_THIN);
        //右边框
        cellStyle2.setBorderRight(HSSFCellStyle.BORDER_THIN);

        //字段样式（垂直居中）
        HSSFCellStyle cellStyle3 = wb.createCellStyle();
        //垂直居中
        cellStyle3.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        //设置自动换行
//        cellStyle3.setWrapText(true);

        //设置边框样式
        //下边框
        cellStyle3.setBorderBottom(HSSFCellStyle.BORDER_THIN);
        //左边框
        cellStyle3.setBorderLeft(HSSFCellStyle.BORDER_THIN);
        //上边框
        cellStyle3.setBorderTop(HSSFCellStyle.BORDER_THIN);
        //右边框
        cellStyle3.setBorderRight(HSSFCellStyle.BORDER_THIN);

        //创建表头
        HSSFRow row = sheet.createRow(0);
        //行高
        row.setHeightInPoints(20);

        HSSFCell cell = row.createCell(0);
        cell.setCellValue(sheetTitle);
        cell.setCellStyle(cellStyle);

        sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, (title.length - 1)));

        //创建标题
        HSSFRow rowTitle = sheet.createRow(1);
        rowTitle.setHeightInPoints(20);

        HSSFCell hc;
        for (int i = 0; i < title.length; i++) {
            hc = rowTitle.createCell(i);
            hc.setCellValue(title[i]);
            hc.setCellStyle(cellStyle2);
        }

        byte result[] = null;

        ByteArrayOutputStream out = null;

        try {
            //创建表格数据
            Field[] fields;
            int i = 2;

            for (Object obj : list) {
                fields = obj.getClass().getDeclaredFields();

                HSSFRow rowBody = sheet.createRow(i);
                rowBody.setHeightInPoints(20);

                int j = 0;
                for (Field f : fields) {

                    f.setAccessible(true);

                    Object value = f.get(obj);
                    if (null == value) {
                        value = "";
                    }

                    hc = rowBody.createCell(j);
                    hc.setCellValue(value.toString());
                    hc.setCellStyle(cellStyle3);

                    j++;
                }

                i++;
            }

            //设置自动列宽
            for (String str : title) {
                if (str.equals("描述")) {
                    //调整第八列宽度(资金明细导出)
                    sheet.autoSizeColumn((short) 7);
                }
            }

            out = new ByteArrayOutputStream();
            wb.write(out);
            result = out.toByteArray();
        } catch (Exception ex) {
            Logger.getLogger(ExcelUtil.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                if (null != out) {
                    out.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(ExcelUtil.class.getName()).log(Level.SEVERE, null, ex);
            }
        }

        return result;
    }

    /**
     * 日期转化为字符串,格式为yyyy-MM-dd-HH-mm-ss
     *
     * @return
     */
    public static String getSystemDate() {
        //得到long类型当前时间
        long l = System.currentTimeMillis();
        //new日期对
        Date date = new Date(l);
        String format = "yyyy-MM-dd-HH-mm-ss";
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        return sdf.format(date);
    }

}
