package com.slodon.b2b2c.controller.statistics.admin;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.slodon.b2b2c.core.response.JsonResult;
import com.slodon.b2b2c.core.response.SldResponse;
import com.slodon.b2b2c.enums.StatsConstant;
import com.slodon.b2b2c.util.PlatformPresentDataUtil;
import com.slodon.b2b2c.vo.statistics.AdminPresentDataVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;

@Api(tags = "admin-实时分析")
@RestController
@RequestMapping("v3/statistics/admin/present/analysis")
public class AdminPresentAnalysisController {

    @Resource
    private RedisTemplate<String, Object> objectRedisTemplate;
    @Resource
    private PlatformPresentDataUtil platformPresentDataUtil;

    @ApiOperation("获取实时统计数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "refresh", value = "true-刷新,默认false", defaultValue = "false")
    })
    @GetMapping("presentData")
    public JsonResult<AdminPresentDataVO> getAdminPresentData(HttpServletRequest request,
                                                              @RequestParam(value = "refresh", required = false, defaultValue = "false") Boolean refresh) {
        if (refresh) {
            //刷新请求，重新计算实时统计，并缓存
            Date endTime = new Date();//截止当前时间
            DateTime startTime = DateUtil.beginOfDay(endTime);//今日0时开始

            //计算实时统计数据
            AdminPresentDataVO vo = platformPresentDataUtil.calculatePresentData(startTime, endTime);

            //缓存数据
            objectRedisTemplate.opsForHash().put(StatsConstant.ADMIN_PRESENT_REDIS_KEY, StatsConstant.ADMIN_PRESENT_REDIS_KEY, vo);

            return SldResponse.success(vo);
        }

        //查询缓存数据
        Object presentData = objectRedisTemplate.opsForHash().get(StatsConstant.ADMIN_PRESENT_REDIS_KEY, StatsConstant.ADMIN_PRESENT_REDIS_KEY);
        if (presentData == null) {
            //无缓存数据，重新计算实时统计，并缓存
            Date endTime = new Date();//截止当前时间
            DateTime startTime = DateUtil.beginOfDay(endTime);//今日0时开始

            //计算实时统计数据
            AdminPresentDataVO vo = platformPresentDataUtil.calculatePresentData(startTime, endTime);

            //缓存数据
            objectRedisTemplate.opsForHash().put(StatsConstant.ADMIN_PRESENT_REDIS_KEY, StatsConstant.ADMIN_PRESENT_REDIS_KEY, vo);

            return SldResponse.success(vo);
        }
        //直接返回缓存
        return SldResponse.success(JSON.parseObject(JSON.toJSONString(presentData), AdminPresentDataVO.class));
    }
}
