package com.slodon.b2b2c.model.goods;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.core.util.AssertUtil;
import com.slodon.b2b2c.dao.read.goods.GoodsParameterGroupReadMapper;
import com.slodon.b2b2c.dao.write.goods.GoodsParameterGroupWriteMapper;
import com.slodon.b2b2c.dao.write.goods.GoodsParameterWriteMapper;
import com.slodon.b2b2c.goods.dto.GoodsParameterGroupAddDTO;
import com.slodon.b2b2c.goods.dto.GoodsParameterGroupUpdateDTO;
import com.slodon.b2b2c.goods.example.GoodsParameterExample;
import com.slodon.b2b2c.goods.example.GoodsParameterGroupExample;
import com.slodon.b2b2c.goods.pojo.GoodsParameterGroup;
import com.slodon.b2b2c.seller.pojo.Vendor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.PropertyUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

@Component
@Slf4j
public class GoodsParameterGroupModel {

    @Resource
    private GoodsParameterGroupReadMapper goodsParameterGroupReadMapper;
    @Resource
    private GoodsParameterGroupWriteMapper goodsParameterGroupWriteMapper;
    @Resource
    private GoodsParameterWriteMapper goodsParameterWriteMapper;

    /**
     * 新增属性分组-店铺自定义商品组参数表，仅作详情顶部的展示使用
     *
     * @param goodsParameterGroup 自定义商品组参数信息
     */
    public void saveGoodsParameterGroup(GoodsParameterGroup goodsParameterGroup) {
        int count = goodsParameterGroupWriteMapper.insert(goodsParameterGroup);
        if (count == 0) {
            throw new MallException("添加属性分组-店铺自定义商品组参数表，仅作详情顶部的展示使用失败，请重试");
        }
    }

    /**
     * 新增属性分组-店铺自定义商品组参数表，仅作详情顶部的展示使用
     *
     * @param goodsParameterGroupAddDTO,vendor 自定义商品组参数信息
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveGoodsParameterGroup(GoodsParameterGroupAddDTO goodsParameterGroupAddDTO, Vendor vendor) throws Exception {
        //查重
        GoodsParameterGroupExample example = new GoodsParameterGroupExample();
        example.setStoreId(vendor.getStoreId());
        example.setGroupName(goodsParameterGroupAddDTO.getGroupName());
        List<GoodsParameterGroup> list = goodsParameterGroupReadMapper.listByExample(example);
        if (!CollectionUtils.isEmpty(list)) {
            throw new MallException("属性分组名称重复,请重新填写");
        }

        //新增
        GoodsParameterGroup insertOne = new GoodsParameterGroup();
        PropertyUtils.copyProperties(insertOne, goodsParameterGroupAddDTO);
        insertOne.setStoreId(vendor.getStoreId());
        insertOne.setCreateVendorId(vendor.getVendorId());
        insertOne.setCreateTime(new Date());
        this.saveGoodsParameterGroup(insertOne);
    }

    /**
     * 根据groupId删除属性分组-店铺自定义商品组参数表，仅作详情顶部的展示使用
     *
     * @param groupId groupId
     * @return
     */
    public Integer deleteGoodsParameterGroup(Integer groupId) {
        if (StringUtils.isEmpty(groupId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = goodsParameterGroupWriteMapper.deleteByPrimaryKey(groupId);
        if (count == 0) {
            log.error("根据groupId：" + groupId + "删除属性分组-店铺自定义商品组参数表，仅作详情顶部的展示使用失败");
            throw new MallException("删除属性分组-店铺自定义商品组参数表，仅作详情顶部的展示使用失败,请重试");
        }
        return count;
    }

    /**
     * 根据groupIds删除属性分组-店铺自定义商品组参数表，仅作详情顶部的展示使用
     *
     * @param groupIds 自定义商品组id
     * @param storeId  店铺id
     */
    @Transactional(rollbackFor = Exception.class)
    public void deleteGoodsParameterGroup(String groupIds, Long storeId) {
        GoodsParameterGroupExample example = new GoodsParameterGroupExample();
        example.setStoreId(storeId);
        example.setGroupIdIn(groupIds);
        List<GoodsParameterGroup> list = goodsParameterGroupReadMapper.listByExample(example);
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        list.forEach(goodsParameterGroup -> {
            AssertUtil.isTrue(!goodsParameterGroup.getStoreId().equals(storeId), "非法操作");
        });

        //删除属性,再删除自定义属性组
        GoodsParameterExample goodsParameterExample = new GoodsParameterExample();
        goodsParameterExample.setStoreId(storeId);
        goodsParameterExample.setGroupIdIn(groupIds);
        goodsParameterWriteMapper.deleteByExample(goodsParameterExample);

        int count = goodsParameterGroupWriteMapper.deleteByExample(example);
        if (count == 0) {
            log.error("根据groupIds：" + groupIds + "删除属性分组-店铺自定义商品组参数表，仅作详情顶部的展示使用失败");
            throw new MallException("删除属性分组-店铺自定义商品组参数表，仅作详情顶部的展示使用失败,请重试");
        }
    }

    /**
     * 根据groupId更新属性分组-店铺自定义商品组参数表，仅作详情顶部的展示使用
     *
     * @param goodsParameterGroup 自定义商品组参数信息
     */
    public void updateGoodsParameterGroup(GoodsParameterGroup goodsParameterGroup) {
        if (StringUtils.isEmpty(goodsParameterGroup.getGroupId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = goodsParameterGroupWriteMapper.updateByPrimaryKeySelective(goodsParameterGroup);
        if (count == 0) {
            log.error("根据groupId：" + goodsParameterGroup.getGroupId() + "更新属性分组-店铺自定义商品组参数表，仅作详情顶部的展示使用失败");
            throw new MallException("更新属性分组-店铺自定义商品组参数表，仅作详情顶部的展示使用失败,请重试");
        }
    }

    /**
     * 根据groupId更新属性分组-店铺自定义商品组参数表，仅作详情顶部的展示使用
     *
     * @param goodsParameterGroupUpdateDTO 前端参数
     */
    @Transactional(rollbackFor = Exception.class)
    public void updateGoodsParameterGroup(GoodsParameterGroupUpdateDTO goodsParameterGroupUpdateDTO, Long storeId) throws Exception {
        //查重
        if (!StringUtils.isEmpty(goodsParameterGroupUpdateDTO.getGroupName())) {
            GoodsParameterGroupExample example = new GoodsParameterGroupExample();
            example.setStoreId(storeId);
            example.setGroupName(goodsParameterGroupUpdateDTO.getGroupName());
            example.setGroupIdNotEquals(goodsParameterGroupUpdateDTO.getGroupId());
            List<GoodsParameterGroup> list = goodsParameterGroupReadMapper.listByExample(example);
            if (!CollectionUtils.isEmpty(list)) {
                throw new MallException("属性分组名称重复,请重新填写");
            }
        }

        //修改
        GoodsParameterGroup updateOne = new GoodsParameterGroup();
        PropertyUtils.copyProperties(updateOne, goodsParameterGroupUpdateDTO);
        this.updateGoodsParameterGroup(updateOne);
    }

    /**
     * 根据groupId获取属性分组-店铺自定义商品组参数表，仅作详情顶部的展示使用详情
     *
     * @param groupId groupId
     * @return
     */
    public GoodsParameterGroup getGoodsParameterGroupByGroupId(Integer groupId) {
        return goodsParameterGroupReadMapper.getByPrimaryKey(groupId);
    }

    /**
     * 根据条件获取属性分组-店铺自定义商品组参数表，仅作详情顶部的展示使用列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<GoodsParameterGroup> getGoodsParameterGroupList(GoodsParameterGroupExample example, PagerInfo pager) {
        List<GoodsParameterGroup> goodsParameterGroupList;
        if (pager != null) {
            pager.setRowsCount(goodsParameterGroupReadMapper.countByExample(example));
            goodsParameterGroupList = goodsParameterGroupReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            goodsParameterGroupList = goodsParameterGroupReadMapper.listByExample(example);
        }
        return goodsParameterGroupList;
    }
}