package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.BrandHourReadMapper;
import com.slodon.b2b2c.dao.write.statistics.BrandHourWriteMapper;
import com.slodon.b2b2c.statistics.example.BrandHourExample;
import com.slodon.b2b2c.statistics.pojo.BrandHour;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 品牌时统计表model
 */
@Component
@Slf4j
public class BrandHourModel {
    @Resource
    private BrandHourReadMapper brandHourReadMapper;

    @Resource
    private BrandHourWriteMapper brandHourWriteMapper;

    /**
     * 新增品牌时统计表
     *
     * @param brandHour
     * @return
     */
    public Integer saveBrandHour(BrandHour brandHour) {
        int count = brandHourWriteMapper.insert(brandHour);
        if (count == 0) {
            throw new MallException("添加品牌时统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除品牌时统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteBrandHour(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = brandHourWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除品牌时统计表失败");
            throw new MallException("删除品牌时统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新品牌时统计表
     *
     * @param brandHour
     * @return
     */
    public Integer updateBrandHour(BrandHour brandHour) {
        if (StringUtils.isEmpty(brandHour.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = brandHourWriteMapper.updateByPrimaryKeySelective(brandHour);
        if (count == 0) {
            log.error("根据statsId：" + brandHour.getStatsId() + "更新品牌时统计表失败");
            throw new MallException("更新品牌时统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取品牌时统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public BrandHour getBrandHourByStatsId(Long statsId) {
        return brandHourReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取品牌时统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<BrandHour> getBrandHourList(BrandHourExample example, PagerInfo pager) {
        List<BrandHour> brandHourList;
        if (pager != null) {
            pager.setRowsCount(brandHourReadMapper.countByExample(example));
            brandHourList = brandHourReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            brandHourList = brandHourReadMapper.listByExample(example);
        }
        return brandHourList;
    }
}