package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.BrandYearReadMapper;
import com.slodon.b2b2c.dao.write.statistics.BrandYearWriteMapper;
import com.slodon.b2b2c.statistics.example.BrandYearExample;
import com.slodon.b2b2c.statistics.pojo.BrandYear;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 品牌年统计表model
 */
@Component
@Slf4j
public class BrandYearModel {
    @Resource
    private BrandYearReadMapper brandYearReadMapper;

    @Resource
    private BrandYearWriteMapper brandYearWriteMapper;

    /**
     * 新增品牌年统计表
     *
     * @param brandYear
     * @return
     */
    public Integer saveBrandYear(BrandYear brandYear) {
        int count = brandYearWriteMapper.insert(brandYear);
        if (count == 0) {
            throw new MallException("添加品牌年统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除品牌年统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteBrandYear(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = brandYearWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除品牌年统计表失败");
            throw new MallException("删除品牌年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新品牌年统计表
     *
     * @param brandYear
     * @return
     */
    public Integer updateBrandYear(BrandYear brandYear) {
        if (StringUtils.isEmpty(brandYear.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = brandYearWriteMapper.updateByPrimaryKeySelective(brandYear);
        if (count == 0) {
            log.error("根据statsId：" + brandYear.getStatsId() + "更新品牌年统计表失败");
            throw new MallException("更新品牌年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取品牌年统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public BrandYear getBrandYearByStatsId(Long statsId) {
        return brandYearReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取品牌年统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<BrandYear> getBrandYearList(BrandYearExample example, PagerInfo pager) {
        List<BrandYear> brandYearList;
        if (pager != null) {
            pager.setRowsCount(brandYearReadMapper.countByExample(example));
            brandYearList = brandYearReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            brandYearList = brandYearReadMapper.listByExample(example);
        }
        return brandYearList;
    }
}