package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.CategoryMonthReadMapper;
import com.slodon.b2b2c.dao.write.statistics.CategoryMonthWriteMapper;
import com.slodon.b2b2c.statistics.example.CategoryMonthExample;
import com.slodon.b2b2c.statistics.pojo.CategoryMonth;
import com.slodon.b2b2c.vo.statistics.SalesTrendVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 商品分类月统计表model
 */
@Component
@Slf4j
public class CategoryMonthModel {

    @Resource
    private CategoryMonthReadMapper categoryMonthReadMapper;
    @Resource
    private CategoryMonthWriteMapper categoryMonthWriteMapper;

    /**
     * 新增商品分类月统计表
     *
     * @param categoryMonth
     * @return
     */
    public Integer saveCategoryMonth(CategoryMonth categoryMonth) {
        int count = categoryMonthWriteMapper.insert(categoryMonth);
        if (count == 0) {
            throw new MallException("添加商品分类月统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除商品分类月统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteCategoryMonth(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = categoryMonthWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除商品分类月统计表失败");
            throw new MallException("删除商品分类月统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新商品分类月统计表
     *
     * @param categoryMonth
     * @return
     */
    public Integer updateCategoryMonth(CategoryMonth categoryMonth) {
        if (StringUtils.isEmpty(categoryMonth.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = categoryMonthWriteMapper.updateByPrimaryKeySelective(categoryMonth);
        if (count == 0) {
            log.error("根据statsId：" + categoryMonth.getStatsId() + "更新商品分类月统计表失败");
            throw new MallException("更新商品分类月统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取商品分类月统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public CategoryMonth getCategoryMonthByStatsId(Long statsId) {
        return categoryMonthReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取商品分类月统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<CategoryMonth> getCategoryMonthList(CategoryMonthExample example, PagerInfo pager) {
        List<CategoryMonth> categoryMonthList;
        if (pager != null) {
            pager.setRowsCount(categoryMonthReadMapper.countByExample(example));
            categoryMonthList = categoryMonthReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            categoryMonthList = categoryMonthReadMapper.listByExample(example);
        }
        return categoryMonthList;
    }

    /**
     * 根据条件获取商品分类趋势统计
     *
     * @param example 查询条件信息
     * @return
     */
    public List<SalesTrendVO> getCategoryTrendList(CategoryMonthExample example) {
        return categoryMonthReadMapper.getCategoryTrendList(example);
    }
}