package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.CategoryWeekReadMapper;
import com.slodon.b2b2c.dao.write.statistics.CategoryWeekWriteMapper;
import com.slodon.b2b2c.statistics.example.CategoryWeekExample;
import com.slodon.b2b2c.statistics.pojo.CategoryWeek;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 商品分类周统计表model
 */
@Component
@Slf4j
public class CategoryWeekModel {
    @Resource
    private CategoryWeekReadMapper categoryWeekReadMapper;

    @Resource
    private CategoryWeekWriteMapper categoryWeekWriteMapper;

    /**
     * 新增商品分类周统计表
     *
     * @param categoryWeek
     * @return
     */
    public Integer saveCategoryWeek(CategoryWeek categoryWeek) {
        int count = categoryWeekWriteMapper.insert(categoryWeek);
        if (count == 0) {
            throw new MallException("添加商品分类周统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除商品分类周统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteCategoryWeek(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = categoryWeekWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除商品分类周统计表失败");
            throw new MallException("删除商品分类周统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新商品分类周统计表
     *
     * @param categoryWeek
     * @return
     */
    public Integer updateCategoryWeek(CategoryWeek categoryWeek) {
        if (StringUtils.isEmpty(categoryWeek.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = categoryWeekWriteMapper.updateByPrimaryKeySelective(categoryWeek);
        if (count == 0) {
            log.error("根据statsId：" + categoryWeek.getStatsId() + "更新商品分类周统计表失败");
            throw new MallException("更新商品分类周统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取商品分类周统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public CategoryWeek getCategoryWeekByStatsId(Long statsId) {
        return categoryWeekReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取商品分类周统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<CategoryWeek> getCategoryWeekList(CategoryWeekExample example, PagerInfo pager) {
        List<CategoryWeek> categoryWeekList;
        if (pager != null) {
            pager.setRowsCount(categoryWeekReadMapper.countByExample(example));
            categoryWeekList = categoryWeekReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            categoryWeekList = categoryWeekReadMapper.listByExample(example);
        }
        return categoryWeekList;
    }
}