package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.CategoryYearReadMapper;
import com.slodon.b2b2c.dao.write.statistics.CategoryYearWriteMapper;
import com.slodon.b2b2c.statistics.example.CategoryYearExample;
import com.slodon.b2b2c.statistics.pojo.CategoryYear;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 商品分类年统计表model
 */
@Component
@Slf4j
public class CategoryYearModel {
    @Resource
    private CategoryYearReadMapper categoryYearReadMapper;

    @Resource
    private CategoryYearWriteMapper categoryYearWriteMapper;

    /**
     * 新增商品分类年统计表
     *
     * @param categoryYear
     * @return
     */
    public Integer saveCategoryYear(CategoryYear categoryYear) {
        int count = categoryYearWriteMapper.insert(categoryYear);
        if (count == 0) {
            throw new MallException("添加商品分类年统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除商品分类年统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteCategoryYear(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = categoryYearWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除商品分类年统计表失败");
            throw new MallException("删除商品分类年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新商品分类年统计表
     *
     * @param categoryYear
     * @return
     */
    public Integer updateCategoryYear(CategoryYear categoryYear) {
        if (StringUtils.isEmpty(categoryYear.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = categoryYearWriteMapper.updateByPrimaryKeySelective(categoryYear);
        if (count == 0) {
            log.error("根据statsId：" + categoryYear.getStatsId() + "更新商品分类年统计表失败");
            throw new MallException("更新商品分类年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取商品分类年统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public CategoryYear getCategoryYearByStatsId(Long statsId) {
        return categoryYearReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取商品分类年统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<CategoryYear> getCategoryYearList(CategoryYearExample example, PagerInfo pager) {
        List<CategoryYear> categoryYearList;
        if (pager != null) {
            pager.setRowsCount(categoryYearReadMapper.countByExample(example));
            categoryYearList = categoryYearReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            categoryYearList = categoryYearReadMapper.listByExample(example);
        }
        return categoryYearList;
    }
}