package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.GoodsHourReadMapper;
import com.slodon.b2b2c.dao.write.statistics.GoodsHourWriteMapper;
import com.slodon.b2b2c.statistics.example.GoodsHourExample;
import com.slodon.b2b2c.statistics.pojo.GoodsHour;
import com.slodon.b2b2c.vo.statistics.SalesTrendVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 商品时统计表model
 */
@Component
@Slf4j
public class GoodsHourModel {

    @Resource
    private GoodsHourReadMapper goodsHourReadMapper;
    @Resource
    private GoodsHourWriteMapper goodsHourWriteMapper;

    /**
     * 新增商品时统计表
     *
     * @param goodsHour
     * @return
     */
    public Integer saveGoodsHour(GoodsHour goodsHour) {
        int count = goodsHourWriteMapper.insert(goodsHour);
        if (count == 0) {
            throw new MallException("添加商品时统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除商品时统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteGoodsHour(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = goodsHourWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除商品时统计表失败");
            throw new MallException("删除商品时统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新商品时统计表
     *
     * @param goodsHour
     * @return
     */
    public Integer updateGoodsHour(GoodsHour goodsHour) {
        if (StringUtils.isEmpty(goodsHour.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = goodsHourWriteMapper.updateByPrimaryKeySelective(goodsHour);
        if (count == 0) {
            log.error("根据statsId：" + goodsHour.getStatsId() + "更新商品时统计表失败");
            throw new MallException("更新商品时统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取商品时统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public GoodsHour getGoodsHourByStatsId(Long statsId) {
        return goodsHourReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取商品时统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<GoodsHour> getGoodsHourList(GoodsHourExample example, PagerInfo pager) {
        List<GoodsHour> goodsHourList;
        if (pager != null) {
            pager.setRowsCount(goodsHourReadMapper.countByExample(example));
            goodsHourList = goodsHourReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            goodsHourList = goodsHourReadMapper.listByExample(example);
        }
        return goodsHourList;
    }

    /**
     * 根据条件获取商品统计
     *
     * @param example 查询条件信息
     * @return
     */
    public List<SalesTrendVO> getGoodsTrendList(GoodsHourExample example) {
        return goodsHourReadMapper.getGoodsTrendList(example);
    }
}