package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.GoodsWeekReadMapper;
import com.slodon.b2b2c.dao.write.statistics.GoodsWeekWriteMapper;
import com.slodon.b2b2c.statistics.example.GoodsWeekExample;
import com.slodon.b2b2c.statistics.pojo.GoodsWeek;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 商品周统计表model
 */
@Component
@Slf4j
public class GoodsWeekModel {
    @Resource
    private GoodsWeekReadMapper goodsWeekReadMapper;

    @Resource
    private GoodsWeekWriteMapper goodsWeekWriteMapper;

    /**
     * 新增商品周统计表
     *
     * @param goodsWeek
     * @return
     */
    public Integer saveGoodsWeek(GoodsWeek goodsWeek) {
        int count = goodsWeekWriteMapper.insert(goodsWeek);
        if (count == 0) {
            throw new MallException("添加商品周统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除商品周统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteGoodsWeek(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = goodsWeekWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除商品周统计表失败");
            throw new MallException("删除商品周统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新商品周统计表
     *
     * @param goodsWeek
     * @return
     */
    public Integer updateGoodsWeek(GoodsWeek goodsWeek) {
        if (StringUtils.isEmpty(goodsWeek.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = goodsWeekWriteMapper.updateByPrimaryKeySelective(goodsWeek);
        if (count == 0) {
            log.error("根据statsId：" + goodsWeek.getStatsId() + "更新商品周统计表失败");
            throw new MallException("更新商品周统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取商品周统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public GoodsWeek getGoodsWeekByStatsId(Long statsId) {
        return goodsWeekReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取商品周统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<GoodsWeek> getGoodsWeekList(GoodsWeekExample example, PagerInfo pager) {
        List<GoodsWeek> goodsWeekList;
        if (pager != null) {
            pager.setRowsCount(goodsWeekReadMapper.countByExample(example));
            goodsWeekList = goodsWeekReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            goodsWeekList = goodsWeekReadMapper.listByExample(example);
        }
        return goodsWeekList;
    }
}