package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.GoodsYearReadMapper;
import com.slodon.b2b2c.dao.write.statistics.GoodsYearWriteMapper;
import com.slodon.b2b2c.statistics.example.GoodsYearExample;
import com.slodon.b2b2c.statistics.pojo.GoodsYear;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 商品年统计表model
 */
@Component
@Slf4j
public class GoodsYearModel {
    @Resource
    private GoodsYearReadMapper goodsYearReadMapper;

    @Resource
    private GoodsYearWriteMapper goodsYearWriteMapper;

    /**
     * 新增商品年统计表
     *
     * @param goodsYear
     * @return
     */
    public Integer saveGoodsYear(GoodsYear goodsYear) {
        int count = goodsYearWriteMapper.insert(goodsYear);
        if (count == 0) {
            throw new MallException("添加商品年统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除商品年统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteGoodsYear(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = goodsYearWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除商品年统计表失败");
            throw new MallException("删除商品年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新商品年统计表
     *
     * @param goodsYear
     * @return
     */
    public Integer updateGoodsYear(GoodsYear goodsYear) {
        if (StringUtils.isEmpty(goodsYear.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = goodsYearWriteMapper.updateByPrimaryKeySelective(goodsYear);
        if (count == 0) {
            log.error("根据statsId：" + goodsYear.getStatsId() + "更新商品年统计表失败");
            throw new MallException("更新商品年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取商品年统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public GoodsYear getGoodsYearByStatsId(Long statsId) {
        return goodsYearReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取商品年统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<GoodsYear> getGoodsYearList(GoodsYearExample example, PagerInfo pager) {
        List<GoodsYear> goodsYearList;
        if (pager != null) {
            pager.setRowsCount(goodsYearReadMapper.countByExample(example));
            goodsYearList = goodsYearReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            goodsYearList = goodsYearReadMapper.listByExample(example);
        }
        return goodsYearList;
    }
}