package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.MemberHourReadMapper;
import com.slodon.b2b2c.dao.write.statistics.MemberHourWriteMapper;
import com.slodon.b2b2c.statistics.example.MemberHourExample;
import com.slodon.b2b2c.statistics.pojo.MemberHour;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 会员时统计表model
 */
@Component
@Slf4j
public class MemberHourModel {
    @Resource
    private MemberHourReadMapper memberHourReadMapper;

    @Resource
    private MemberHourWriteMapper memberHourWriteMapper;

    /**
     * 新增会员时统计表
     *
     * @param memberHour
     * @return
     */
    public Integer saveMemberHour(MemberHour memberHour) {
        int count = memberHourWriteMapper.insert(memberHour);
        if (count == 0) {
            throw new MallException("添加会员时统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除会员时统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteMemberHour(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = memberHourWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除会员时统计表失败");
            throw new MallException("删除会员时统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新会员时统计表
     *
     * @param memberHour
     * @return
     */
    public Integer updateMemberHour(MemberHour memberHour) {
        if (StringUtils.isEmpty(memberHour.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = memberHourWriteMapper.updateByPrimaryKeySelective(memberHour);
        if (count == 0) {
            log.error("根据statsId：" + memberHour.getStatsId() + "更新会员时统计表失败");
            throw new MallException("更新会员时统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取会员时统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public MemberHour getMemberHourByStatsId(Long statsId) {
        return memberHourReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取会员时统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<MemberHour> getMemberHourList(MemberHourExample example, PagerInfo pager) {
        List<MemberHour> memberHourList;
        if (pager != null) {
            pager.setRowsCount(memberHourReadMapper.countByExample(example));
            memberHourList = memberHourReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            memberHourList = memberHourReadMapper.listByExample(example);
        }
        return memberHourList;
    }
}