package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.MemberMonthReadMapper;
import com.slodon.b2b2c.dao.write.statistics.MemberMonthWriteMapper;
import com.slodon.b2b2c.statistics.example.MemberMonthExample;
import com.slodon.b2b2c.statistics.pojo.MemberMonth;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 会员月统计表model
 */
@Component
@Slf4j
public class MemberMonthModel {
    @Resource
    private MemberMonthReadMapper memberMonthReadMapper;

    @Resource
    private MemberMonthWriteMapper memberMonthWriteMapper;

    /**
     * 新增会员月统计表
     *
     * @param memberMonth
     * @return
     */
    public Integer saveMemberMonth(MemberMonth memberMonth) {
        int count = memberMonthWriteMapper.insert(memberMonth);
        if (count == 0) {
            throw new MallException("添加会员月统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除会员月统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteMemberMonth(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = memberMonthWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除会员月统计表失败");
            throw new MallException("删除会员月统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新会员月统计表
     *
     * @param memberMonth
     * @return
     */
    public Integer updateMemberMonth(MemberMonth memberMonth) {
        if (StringUtils.isEmpty(memberMonth.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = memberMonthWriteMapper.updateByPrimaryKeySelective(memberMonth);
        if (count == 0) {
            log.error("根据statsId：" + memberMonth.getStatsId() + "更新会员月统计表失败");
            throw new MallException("更新会员月统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取会员月统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public MemberMonth getMemberMonthByStatsId(Long statsId) {
        return memberMonthReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取会员月统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<MemberMonth> getMemberMonthList(MemberMonthExample example, PagerInfo pager) {
        List<MemberMonth> memberMonthList;
        if (pager != null) {
            pager.setRowsCount(memberMonthReadMapper.countByExample(example));
            memberMonthList = memberMonthReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            memberMonthList = memberMonthReadMapper.listByExample(example);
        }
        return memberMonthList;
    }
}