package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.MemberStoreHourReadMapper;
import com.slodon.b2b2c.dao.write.statistics.MemberStoreHourWriteMapper;
import com.slodon.b2b2c.statistics.example.MemberStoreHourExample;
import com.slodon.b2b2c.statistics.pojo.MemberStoreHour;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 店铺会员时统计表model
 */
@Component
@Slf4j
public class MemberStoreHourModel {
    @Resource
    private MemberStoreHourReadMapper memberStoreHourReadMapper;

    @Resource
    private MemberStoreHourWriteMapper memberStoreHourWriteMapper;

    /**
     * 新增店铺会员时统计表
     *
     * @param memberStoreHour
     * @return
     */
    public Integer saveMemberStoreHour(MemberStoreHour memberStoreHour) {
        int count = memberStoreHourWriteMapper.insert(memberStoreHour);
        if (count == 0) {
            throw new MallException("添加店铺会员时统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除店铺会员时统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteMemberStoreHour(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = memberStoreHourWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除店铺会员时统计表失败");
            throw new MallException("删除店铺会员时统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新店铺会员时统计表
     *
     * @param memberStoreHour
     * @return
     */
    public Integer updateMemberStoreHour(MemberStoreHour memberStoreHour) {
        if (StringUtils.isEmpty(memberStoreHour.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = memberStoreHourWriteMapper.updateByPrimaryKeySelective(memberStoreHour);
        if (count == 0) {
            log.error("根据statsId：" + memberStoreHour.getStatsId() + "更新店铺会员时统计表失败");
            throw new MallException("更新店铺会员时统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取店铺会员时统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public MemberStoreHour getMemberStoreHourByStatsId(Long statsId) {
        return memberStoreHourReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取店铺会员时统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<MemberStoreHour> getMemberStoreHourList(MemberStoreHourExample example, PagerInfo pager) {
        List<MemberStoreHour> memberStoreHourList;
        if (pager != null) {
            pager.setRowsCount(memberStoreHourReadMapper.countByExample(example));
            memberStoreHourList = memberStoreHourReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            memberStoreHourList = memberStoreHourReadMapper.listByExample(example);
        }
        return memberStoreHourList;
    }
}