package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.MemberStoreYearReadMapper;
import com.slodon.b2b2c.dao.write.statistics.MemberStoreYearWriteMapper;
import com.slodon.b2b2c.statistics.example.MemberStoreYearExample;
import com.slodon.b2b2c.statistics.pojo.MemberStoreYear;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 店铺会员年统计表model
 */
@Component
@Slf4j
public class MemberStoreYearModel {
    @Resource
    private MemberStoreYearReadMapper memberStoreYearReadMapper;

    @Resource
    private MemberStoreYearWriteMapper memberStoreYearWriteMapper;

    /**
     * 新增店铺会员年统计表
     *
     * @param memberStoreYear
     * @return
     */
    public Integer saveMemberStoreYear(MemberStoreYear memberStoreYear) {
        int count = memberStoreYearWriteMapper.insert(memberStoreYear);
        if (count == 0) {
            throw new MallException("添加店铺会员年统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除店铺会员年统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteMemberStoreYear(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = memberStoreYearWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除店铺会员年统计表失败");
            throw new MallException("删除店铺会员年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新店铺会员年统计表
     *
     * @param memberStoreYear
     * @return
     */
    public Integer updateMemberStoreYear(MemberStoreYear memberStoreYear) {
        if (StringUtils.isEmpty(memberStoreYear.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = memberStoreYearWriteMapper.updateByPrimaryKeySelective(memberStoreYear);
        if (count == 0) {
            log.error("根据statsId：" + memberStoreYear.getStatsId() + "更新店铺会员年统计表失败");
            throw new MallException("更新店铺会员年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取店铺会员年统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public MemberStoreYear getMemberStoreYearByStatsId(Long statsId) {
        return memberStoreYearReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取店铺会员年统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<MemberStoreYear> getMemberStoreYearList(MemberStoreYearExample example, PagerInfo pager) {
        List<MemberStoreYear> memberStoreYearList;
        if (pager != null) {
            pager.setRowsCount(memberStoreYearReadMapper.countByExample(example));
            memberStoreYearList = memberStoreYearReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            memberStoreYearList = memberStoreYearReadMapper.listByExample(example);
        }
        return memberStoreYearList;
    }
}