package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.MemberYearReadMapper;
import com.slodon.b2b2c.dao.write.statistics.MemberYearWriteMapper;
import com.slodon.b2b2c.statistics.example.MemberYearExample;
import com.slodon.b2b2c.statistics.pojo.MemberYear;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 会员年统计表model
 */
@Component
@Slf4j
public class MemberYearModel {
    @Resource
    private MemberYearReadMapper memberYearReadMapper;

    @Resource
    private MemberYearWriteMapper memberYearWriteMapper;

    /**
     * 新增会员年统计表
     *
     * @param memberYear
     * @return
     */
    public Integer saveMemberYear(MemberYear memberYear) {
        int count = memberYearWriteMapper.insert(memberYear);
        if (count == 0) {
            throw new MallException("添加会员年统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除会员年统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteMemberYear(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = memberYearWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除会员年统计表失败");
            throw new MallException("删除会员年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新会员年统计表
     *
     * @param memberYear
     * @return
     */
    public Integer updateMemberYear(MemberYear memberYear) {
        if (StringUtils.isEmpty(memberYear.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = memberYearWriteMapper.updateByPrimaryKeySelective(memberYear);
        if (count == 0) {
            log.error("根据statsId：" + memberYear.getStatsId() + "更新会员年统计表失败");
            throw new MallException("更新会员年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取会员年统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public MemberYear getMemberYearByStatsId(Long statsId) {
        return memberYearReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取会员年统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<MemberYear> getMemberYearList(MemberYearExample example, PagerInfo pager) {
        List<MemberYear> memberYearList;
        if (pager != null) {
            pager.setRowsCount(memberYearReadMapper.countByExample(example));
            memberYearList = memberYearReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            memberYearList = memberYearReadMapper.listByExample(example);
        }
        return memberYearList;
    }
}