package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.PlatformMonthReadMapper;
import com.slodon.b2b2c.dao.write.statistics.PlatformMonthWriteMapper;
import com.slodon.b2b2c.statistics.example.PlatformMonthExample;
import com.slodon.b2b2c.statistics.pojo.PlatformMonth;
import com.slodon.b2b2c.vo.statistics.MovableGoodsVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 平台月统计表model
 */
@Component
@Slf4j
public class PlatformMonthModel {

    @Resource
    private PlatformMonthReadMapper platformMonthReadMapper;
    @Resource
    private PlatformMonthWriteMapper platformMonthWriteMapper;

    /**
     * 新增平台月统计表
     *
     * @param platformMonth
     * @return
     */
    public Integer savePlatformMonth(PlatformMonth platformMonth) {
        int count = platformMonthWriteMapper.insert(platformMonth);
        if (count == 0) {
            throw new MallException("添加平台月统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除平台月统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deletePlatformMonth(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = platformMonthWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除平台月统计表失败");
            throw new MallException("删除平台月统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新平台月统计表
     *
     * @param platformMonth
     * @return
     */
    public Integer updatePlatformMonth(PlatformMonth platformMonth) {
        if (StringUtils.isEmpty(platformMonth.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = platformMonthWriteMapper.updateByPrimaryKeySelective(platformMonth);
        if (count == 0) {
            log.error("根据statsId：" + platformMonth.getStatsId() + "更新平台月统计表失败");
            throw new MallException("更新平台月统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取平台月统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public PlatformMonth getPlatformMonthByStatsId(Long statsId) {
        return platformMonthReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取平台月统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<PlatformMonth> getPlatformMonthList(PlatformMonthExample example, PagerInfo pager) {
        List<PlatformMonth> platformMonthList;
        if (pager != null) {
            pager.setRowsCount(platformMonthReadMapper.countByExample(example));
            platformMonthList = platformMonthReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            platformMonthList = platformMonthReadMapper.listByExample(example);
        }
        return platformMonthList;
    }

    /**
     * 根据条件获取平台趋势统计
     *
     * @param example 查询条件信息
     * @return
     */
    public List<MovableGoodsVO> getPlatformTrendList(PlatformMonthExample example) {
        return platformMonthReadMapper.getPlatformTrendList(example);
    }
}