package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.PlatformYearReadMapper;
import com.slodon.b2b2c.dao.write.statistics.PlatformYearWriteMapper;
import com.slodon.b2b2c.statistics.example.PlatformYearExample;
import com.slodon.b2b2c.statistics.pojo.PlatformYear;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 平台年统计表model
 */
@Component
@Slf4j
public class PlatformYearModel {
    @Resource
    private PlatformYearReadMapper platformYearReadMapper;

    @Resource
    private PlatformYearWriteMapper platformYearWriteMapper;

    /**
     * 新增平台年统计表
     *
     * @param platformYear
     * @return
     */
    public Integer savePlatformYear(PlatformYear platformYear) {
        int count = platformYearWriteMapper.insert(platformYear);
        if (count == 0) {
            throw new MallException("添加平台年统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除平台年统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deletePlatformYear(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = platformYearWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除平台年统计表失败");
            throw new MallException("删除平台年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新平台年统计表
     *
     * @param platformYear
     * @return
     */
    public Integer updatePlatformYear(PlatformYear platformYear) {
        if (StringUtils.isEmpty(platformYear.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = platformYearWriteMapper.updateByPrimaryKeySelective(platformYear);
        if (count == 0) {
            log.error("根据statsId：" + platformYear.getStatsId() + "更新平台年统计表失败");
            throw new MallException("更新平台年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取平台年统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public PlatformYear getPlatformYearByStatsId(Long statsId) {
        return platformYearReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取平台年统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<PlatformYear> getPlatformYearList(PlatformYearExample example, PagerInfo pager) {
        List<PlatformYear> platformYearList;
        if (pager != null) {
            pager.setRowsCount(platformYearReadMapper.countByExample(example));
            platformYearList = platformYearReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            platformYearList = platformYearReadMapper.listByExample(example);
        }
        return platformYearList;
    }
}