package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.ProvinceWeekReadMapper;
import com.slodon.b2b2c.dao.write.statistics.ProvinceWeekWriteMapper;
import com.slodon.b2b2c.statistics.example.ProvinceWeekExample;
import com.slodon.b2b2c.statistics.pojo.ProvinceWeek;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 省份周统计表model
 */
@Component
@Slf4j
public class ProvinceWeekModel {
    @Resource
    private ProvinceWeekReadMapper provinceWeekReadMapper;

    @Resource
    private ProvinceWeekWriteMapper provinceWeekWriteMapper;

    /**
     * 新增省份周统计表
     *
     * @param provinceWeek
     * @return
     */
    public Integer saveProvinceWeek(ProvinceWeek provinceWeek) {
        int count = provinceWeekWriteMapper.insert(provinceWeek);
        if (count == 0) {
            throw new MallException("添加省份周统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除省份周统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteProvinceWeek(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = provinceWeekWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除省份周统计表失败");
            throw new MallException("删除省份周统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新省份周统计表
     *
     * @param provinceWeek
     * @return
     */
    public Integer updateProvinceWeek(ProvinceWeek provinceWeek) {
        if (StringUtils.isEmpty(provinceWeek.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = provinceWeekWriteMapper.updateByPrimaryKeySelective(provinceWeek);
        if (count == 0) {
            log.error("根据statsId：" + provinceWeek.getStatsId() + "更新省份周统计表失败");
            throw new MallException("更新省份周统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取省份周统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public ProvinceWeek getProvinceWeekByStatsId(Long statsId) {
        return provinceWeekReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取省份周统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<ProvinceWeek> getProvinceWeekList(ProvinceWeekExample example, PagerInfo pager) {
        List<ProvinceWeek> provinceWeekList;
        if (pager != null) {
            pager.setRowsCount(provinceWeekReadMapper.countByExample(example));
            provinceWeekList = provinceWeekReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            provinceWeekList = provinceWeekReadMapper.listByExample(example);
        }
        return provinceWeekList;
    }
}