package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.ProvinceYearReadMapper;
import com.slodon.b2b2c.dao.write.statistics.ProvinceYearWriteMapper;
import com.slodon.b2b2c.statistics.example.ProvinceYearExample;
import com.slodon.b2b2c.statistics.pojo.ProvinceYear;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 省份年统计表model
 */
@Component
@Slf4j
public class ProvinceYearModel {
    @Resource
    private ProvinceYearReadMapper provinceYearReadMapper;

    @Resource
    private ProvinceYearWriteMapper provinceYearWriteMapper;

    /**
     * 新增省份年统计表
     *
     * @param provinceYear
     * @return
     */
    public Integer saveProvinceYear(ProvinceYear provinceYear) {
        int count = provinceYearWriteMapper.insert(provinceYear);
        if (count == 0) {
            throw new MallException("添加省份年统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除省份年统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteProvinceYear(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = provinceYearWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除省份年统计表失败");
            throw new MallException("删除省份年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新省份年统计表
     *
     * @param provinceYear
     * @return
     */
    public Integer updateProvinceYear(ProvinceYear provinceYear) {
        if (StringUtils.isEmpty(provinceYear.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = provinceYearWriteMapper.updateByPrimaryKeySelective(provinceYear);
        if (count == 0) {
            log.error("根据statsId：" + provinceYear.getStatsId() + "更新省份年统计表失败");
            throw new MallException("更新省份年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取省份年统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public ProvinceYear getProvinceYearByStatsId(Long statsId) {
        return provinceYearReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取省份年统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<ProvinceYear> getProvinceYearList(ProvinceYearExample example, PagerInfo pager) {
        List<ProvinceYear> provinceYearList;
        if (pager != null) {
            pager.setRowsCount(provinceYearReadMapper.countByExample(example));
            provinceYearList = provinceYearReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            provinceYearList = provinceYearReadMapper.listByExample(example);
        }
        return provinceYearList;
    }
}