package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.StoreMonthReadMapper;
import com.slodon.b2b2c.dao.write.statistics.StoreMonthWriteMapper;
import com.slodon.b2b2c.statistics.example.StoreMonthExample;
import com.slodon.b2b2c.statistics.pojo.StoreMonth;
import com.slodon.b2b2c.vo.statistics.MovableGoodsVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 店铺月统计表model
 */
@Component
@Slf4j
public class StoreMonthModel {

    @Resource
    private StoreMonthReadMapper storeMonthReadMapper;
    @Resource
    private StoreMonthWriteMapper storeMonthWriteMapper;

    /**
     * 新增店铺月统计表
     *
     * @param storeMonth
     * @return
     */
    public Integer saveStoreMonth(StoreMonth storeMonth) {
        int count = storeMonthWriteMapper.insert(storeMonth);
        if (count == 0) {
            throw new MallException("添加店铺月统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除店铺月统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteStoreMonth(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = storeMonthWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除店铺月统计表失败");
            throw new MallException("删除店铺月统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新店铺月统计表
     *
     * @param storeMonth
     * @return
     */
    public Integer updateStoreMonth(StoreMonth storeMonth) {
        if (StringUtils.isEmpty(storeMonth.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = storeMonthWriteMapper.updateByPrimaryKeySelective(storeMonth);
        if (count == 0) {
            log.error("根据statsId：" + storeMonth.getStatsId() + "更新店铺月统计表失败");
            throw new MallException("更新店铺月统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取店铺月统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public StoreMonth getStoreMonthByStatsId(Long statsId) {
        return storeMonthReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取店铺月统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<StoreMonth> getStoreMonthList(StoreMonthExample example, PagerInfo pager) {
        List<StoreMonth> storeMonthList;
        if (pager != null) {
            pager.setRowsCount(storeMonthReadMapper.countByExample(example));
            storeMonthList = storeMonthReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            storeMonthList = storeMonthReadMapper.listByExample(example);
        }
        return storeMonthList;
    }

    /**
     * 根据条件获取店铺统计
     *
     * @param example 查询条件信息
     * @return
     */
    public List<MovableGoodsVO> getStoreTrendList(StoreMonthExample example) {
        return storeMonthReadMapper.getStoreTrendList(example);
    }
}