package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.StoreYearReadMapper;
import com.slodon.b2b2c.dao.write.statistics.StoreYearWriteMapper;
import com.slodon.b2b2c.statistics.example.StoreYearExample;
import com.slodon.b2b2c.statistics.pojo.StoreYear;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 店铺年统计表model
 */
@Component
@Slf4j
public class StoreYearModel {
    @Resource
    private StoreYearReadMapper storeYearReadMapper;

    @Resource
    private StoreYearWriteMapper storeYearWriteMapper;

    /**
     * 新增店铺年统计表
     *
     * @param storeYear
     * @return
     */
    public Integer saveStoreYear(StoreYear storeYear) {
        int count = storeYearWriteMapper.insert(storeYear);
        if (count == 0) {
            throw new MallException("添加店铺年统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除店铺年统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteStoreYear(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = storeYearWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除店铺年统计表失败");
            throw new MallException("删除店铺年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新店铺年统计表
     *
     * @param storeYear
     * @return
     */
    public Integer updateStoreYear(StoreYear storeYear) {
        if (StringUtils.isEmpty(storeYear.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = storeYearWriteMapper.updateByPrimaryKeySelective(storeYear);
        if (count == 0) {
            log.error("根据statsId：" + storeYear.getStatsId() + "更新店铺年统计表失败");
            throw new MallException("更新店铺年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取店铺年统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public StoreYear getStoreYearByStatsId(Long statsId) {
        return storeYearReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取店铺年统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<StoreYear> getStoreYearList(StoreYearExample example, PagerInfo pager) {
        List<StoreYear> storeYearList;
        if (pager != null) {
            pager.setRowsCount(storeYearReadMapper.countByExample(example));
            storeYearList = storeYearReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            storeYearList = storeYearReadMapper.listByExample(example);
        }
        return storeYearList;
    }
}