package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.TerminalMonthReadMapper;
import com.slodon.b2b2c.dao.write.statistics.TerminalMonthWriteMapper;
import com.slodon.b2b2c.statistics.example.TerminalMonthExample;
import com.slodon.b2b2c.statistics.pojo.TerminalMonth;
import com.slodon.b2b2c.vo.statistics.SalesTrendVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 终端月统计表model
 */
@Component
@Slf4j
public class TerminalMonthModel {

    @Resource
    private TerminalMonthReadMapper terminalMonthReadMapper;
    @Resource
    private TerminalMonthWriteMapper terminalMonthWriteMapper;

    /**
     * 新增终端月统计表
     *
     * @param terminalMonth
     * @return
     */
    public Integer saveTerminalMonth(TerminalMonth terminalMonth) {
        int count = terminalMonthWriteMapper.insert(terminalMonth);
        if (count == 0) {
            throw new MallException("添加终端月统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除终端月统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteTerminalMonth(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = terminalMonthWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除终端月统计表失败");
            throw new MallException("删除终端月统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新终端月统计表
     *
     * @param terminalMonth
     * @return
     */
    public Integer updateTerminalMonth(TerminalMonth terminalMonth) {
        if (StringUtils.isEmpty(terminalMonth.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = terminalMonthWriteMapper.updateByPrimaryKeySelective(terminalMonth);
        if (count == 0) {
            log.error("根据statsId：" + terminalMonth.getStatsId() + "更新终端月统计表失败");
            throw new MallException("更新终端月统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取终端月统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public TerminalMonth getTerminalMonthByStatsId(Long statsId) {
        return terminalMonthReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取终端月统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<TerminalMonth> getTerminalMonthList(TerminalMonthExample example, PagerInfo pager) {
        List<TerminalMonth> terminalMonthList;
        if (pager != null) {
            pager.setRowsCount(terminalMonthReadMapper.countByExample(example));
            terminalMonthList = terminalMonthReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            terminalMonthList = terminalMonthReadMapper.listByExample(example);
        }
        return terminalMonthList;
    }

    /**
     * 根据条件获取终端趋势统计
     *
     * @param example 查询条件信息
     * @return
     */
    public List<SalesTrendVO> getTerminalTrendList(TerminalMonthExample example) {
        return terminalMonthReadMapper.getTerminalTrendList(example);
    }
}