package com.slodon.b2b2c.model.statistics;

import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.dao.read.statistics.TerminalYearReadMapper;
import com.slodon.b2b2c.dao.write.statistics.TerminalYearWriteMapper;
import com.slodon.b2b2c.statistics.example.TerminalYearExample;
import com.slodon.b2b2c.statistics.pojo.TerminalYear;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 终端年统计表model
 */
@Component
@Slf4j
public class TerminalYearModel {
    @Resource
    private TerminalYearReadMapper terminalYearReadMapper;

    @Resource
    private TerminalYearWriteMapper terminalYearWriteMapper;

    /**
     * 新增终端年统计表
     *
     * @param terminalYear
     * @return
     */
    public Integer saveTerminalYear(TerminalYear terminalYear) {
        int count = terminalYearWriteMapper.insert(terminalYear);
        if (count == 0) {
            throw new MallException("添加终端年统计表失败，请重试");
        }
        return count;
    }

    /**
     * 根据statsId删除终端年统计表
     *
     * @param statsId statsId
     * @return
     */
    public Integer deleteTerminalYear(Long statsId) {
        if (StringUtils.isEmpty(statsId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = terminalYearWriteMapper.deleteByPrimaryKey(statsId);
        if (count == 0) {
            log.error("根据statsId：" + statsId + "删除终端年统计表失败");
            throw new MallException("删除终端年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId更新终端年统计表
     *
     * @param terminalYear
     * @return
     */
    public Integer updateTerminalYear(TerminalYear terminalYear) {
        if (StringUtils.isEmpty(terminalYear.getStatsId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = terminalYearWriteMapper.updateByPrimaryKeySelective(terminalYear);
        if (count == 0) {
            log.error("根据statsId：" + terminalYear.getStatsId() + "更新终端年统计表失败");
            throw new MallException("更新终端年统计表失败,请重试");
        }
        return count;
    }

    /**
     * 根据statsId获取终端年统计表详情
     *
     * @param statsId statsId
     * @return
     */
    public TerminalYear getTerminalYearByStatsId(Long statsId) {
        return terminalYearReadMapper.getByPrimaryKey(statsId);
    }

    /**
     * 根据条件获取终端年统计表列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<TerminalYear> getTerminalYearList(TerminalYearExample example, PagerInfo pager) {
        List<TerminalYear> terminalYearList;
        if (pager != null) {
            pager.setRowsCount(terminalYearReadMapper.countByExample(example));
            terminalYearList = terminalYearReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            terminalYearList = terminalYearReadMapper.listByExample(example);
        }
        return terminalYearList;
    }
}